@@ -1,6 +1,6 @@
 
 
-### Copyright 2001-2003 Deepayan Sarkar <deepayan@stat.wisc.edu>
+### Copyright (C) 2001-2005 Deepayan Sarkar <Deepayan.Sarkar@R-project.org>
 ###
 ### This file is part of the lattice library for R.
 ### It is made available under the terms of the GNU General Public
@@ -50,11 +50,17 @@ prepanel.default.bwplot <-
             }
             list(xlim =
                  if (stack) {
-                     foo1 <- if (any(x > 0)) range( by(x[x>0], y[x>0, drop = TRUE], sum)) else 0
-                     foo2 <- if (any(x < 0)) range( by(x[x<0], y[x<0, drop = TRUE], sum)) else 0
+                     foo1 <-
+                         if (any(x > 0))
+                             range( by(x[x>0], y[x>0, drop = TRUE], sum), finite = TRUE) 
+                         else 0
+                     foo2 <-
+                         if (any(x < 0))
+                             range( by(x[x<0], y[x<0, drop = TRUE], sum), finite = TRUE) 
+                         else 0
                      range(foo1, foo2)
                  }
-                 else if (is.numeric(x)) range(x[is.finite(x)], origin)
+                 else if (is.numeric(x)) range(x, origin, finite = TRUE)
                  else levels(x),
                  ylim = levels(y),
                  yat = sort(unique(as.numeric(y))),
@@ -72,11 +78,17 @@ prepanel.default.bwplot <-
                  xat = sort(unique(as.numeric(x))),
                  ylim =
                  if (stack) {
-                     foo1 <- if (any(y > 0)) range( by(y[y>0], x[y>0], sum)) else 0
-                     foo2 <- if (any(y < 0)) range( by(y[y<0], x[y<0], sum)) else 0
+                     foo1 <-
+                         if (any(y > 0))
+                             range( by(y[y>0], x[y>0], sum), finite = TRUE)
+                         else 0
+                     foo2 <-
+                         if (any(y < 0))
+                             range( by(y[y<0], x[y<0], sum, finite = TRUE)) 
+                         else 0
                      range(foo1, foo2)
                  }
-                 else if (is.numeric(y)) range(y[is.finite(y)], origin)
+                 else if (is.numeric(y)) range(y, origin, finite = TRUE)
                  else levels(y),
                  dx = 1,
                  dy = 1)
@@ -342,8 +354,6 @@ panel.barchart <-
                               just = c("centre", "bottom"),
                               default.units = "native")
             }
-
-            
         }
         else
         {
@@ -468,13 +478,17 @@ panel.stripplot <-
 
 
 panel.bwplot <-
-    function(x, y, box.ratio=1, horizontal = TRUE, pch=box.dot$pch,
-             col = box.dot$col, cex = box.dot$cex,
+    function(x, y, box.ratio = 1,
+             horizontal = TRUE,
+             pch = box.dot$pch,
+             col = box.dot$col,
+             cex = box.dot$cex,
              font = box.dot$font,
              fontfamily = box.dot$fontfamily,
              fontface = box.dot$fontface, 
-             fill = box.rectangle$fill, varwidth = FALSE,
-             levels.fos = if (horizontal) unique(y) else unique(x),
+             fill = box.rectangle$fill,
+             varwidth = FALSE,
+             levels.fos = if (horizontal) sort(unique(y)) else sort(unique(x)),
              coef = 1.5, do.out = TRUE, ...)
 {
     x <- as.numeric(x)
@@ -488,167 +502,486 @@ panel.bwplot <-
     plot.symbol <- trellis.par.get("plot.symbol")
 
     fontsize.points <- trellis.par.get("fontsize")$points
-
-    ## In case levels.fos is not given (which should not happen), I'll
-    ## be working on the premise that EACH INTEGER in the y-RANGE is a
-    ## potential location of a boxplot. 
+    cur.limits <- current.panel.limits()
+    xscale <- cur.limits$xlim
+    yscale <- cur.limits$ylim
 
     if (horizontal)
     {
-
-        maxn <- max(by(x, y, length)) ## used if varwidth = TRUE
-
-        yscale <- current.viewport()$yscale
-        lower <- ceiling(yscale[1])
-        height <- box.ratio/(1+box.ratio)
-        xscale <- current.viewport()$xscale
-
-        for (yval in levels.fos)
+        blist <-
+            tapply(x, factor(y, levels = levels.fos),
+                   boxplot.stats,
+                   coef = coef,
+                   do.out = do.out)
+        blist.stats <- t(sapply(blist, "[[", "stats"))
+        blist.out <- lapply(blist, "[[", "out")
+        blist.height <- box.ratio / (1 + box.ratio)
+        if (varwidth)
         {
-
-            ## yval  <- i
-            stats <- boxplot.stats(x[y==yval], coef = coef, do.out = do.out)
-            
-            if (stats$n>0)
-            {
-                pushViewport(viewport(y=unit(yval, "native"),
-                                      height = unit((if (varwidth)
-                                      sqrt(stats$n/maxn)  else 1) * height, "native"),
-                                      xscale = xscale))
-                
-                r.x <- (stats$stats[2]+stats$stats[4])/2
-                r.w <- stats$stats[4]-stats$stats[2]
-                grid.rect(x = unit(r.x, "native"), width = unit(r.w, "native"),
-                          gp = gpar(lwd = box.rectangle$lwd,
-                          lty = box.rectangle$lty,
-                          fill = fill,
-                          col = box.rectangle$col))
-                
-                grid.lines(x = unit(stats$stats[1:2],"native"),
-                           y=unit(c(.5,.5), "npc"),
-                           gp = gpar(col = box.umbrella$col,
-                           lwd = box.umbrella$lwd,
-                           lty = box.umbrella$lty))
-                
-                grid.lines(x = unit(stats$stats[4:5],"native"),
-                           y=unit(c(.5,.5), "npc"),
-                           gp = gpar(col = box.umbrella$col,
-                           lwd = box.umbrella$lwd,
-                           lty = box.umbrella$lty))
-                
-                grid.lines(x = unit(rep(stats$stats[1],2),"native"),
-                           y=unit(c(0,1), "npc"),
-                           gp = gpar(col = box.umbrella$col,
-                           lwd = box.umbrella$lwd,
-                           lty = box.umbrella$lty))
-                
-                grid.lines(x = unit(rep(stats$stats[5],2),"native"),
-                           y=unit(c(0,1), "npc"),
-                           gp = gpar(col = box.umbrella$col,
-                           lwd = box.umbrella$lwd,
-                           lty = box.umbrella$lty))
-                
-                grid.points(x = stats$stats[3], y = .5, pch = pch, 
-                            gp =
-                            gpar(col = col, cex = cex,
-                                 fontfamily = fontfamily,
-                                 fontface = chooseFace(fontface, font),
-                                 fontsize = fontsize.points))
-                
-                if ((l<-length(stats$out))>0)
-                    grid.points(x = stats$out, y = rep(.5,l),
-                                pch = plot.symbol$pch,
-                                gp =
-                                gpar(col = plot.symbol$col,
-                                     cex = plot.symbol$cex,
-                                     fontfamily = plot.symbol$fontfamily,
-                                     fontface = chooseFace(plot.symbol$fontface, plot.symbol$font),
-                                     fontsize = fontsize.points))
-                
-                popViewport()
-                
-            }
+            maxn <- max(by(x, y, length))
+            blist.n <- sapply(blist, "[[", "n")
+            blist.height <- sqrt(blist.n / maxn) * blist.height
         }
-        
+
+        ## box
+
+        panel.rect(xleft = blist.stats[, 2],
+                   xright = blist.stats[, 4],
+                   y = levels.fos, 
+                   height = blist.height,
+                   lwd = box.rectangle$lwd,
+                   lty = box.rectangle$lty,
+                   fill = fill,
+                   col = box.rectangle$col)
+
+        ## whiskers
+
+        panel.segments(c(blist.stats[, 2], blist.stats[, 4]),
+                       rep(levels.fos, 2),
+                       c(blist.stats[, 1], blist.stats[, 5]),
+                       rep(levels.fos, 2),
+                       col = box.umbrella$col,
+                       lwd = box.umbrella$lwd,
+                       lty = box.umbrella$lty)
+        panel.segments(c(blist.stats[, 1], blist.stats[, 5]),
+                       levels.fos - blist.height / 2,
+                       c(blist.stats[, 1], blist.stats[, 5]),
+                       levels.fos + blist.height / 2,
+                       col = box.umbrella$col,
+                       lwd = box.umbrella$lwd,
+                       lty = box.umbrella$lty)
+
+        ## dot
+
+        if (all(pch == "|"))
+        {
+            panel.segments(blist.stats[, 3],
+                           levels.fos - blist.height / 2,
+                           blist.stats[, 3],
+                           levels.fos + blist.height / 2,
+                           lwd = box.rectangle$lwd,
+                           lty = box.rectangle$lty,
+                           col = box.rectangle$col)
+        }
+        else
+        {
+            panel.points(x = blist.stats[, 3],
+                         y = levels.fos,
+                         pch = pch,
+                         col = col, cex = cex,
+                         fontfamily = fontfamily,
+                         fontface = chooseFace(fontface, font),
+                         fontsize = fontsize.points)
+        }
+
+        ## outliers
+
+        panel.points(x = unlist(blist.out),
+                     y = rep(levels.fos, sapply(blist.out, length)),
+                     pch = plot.symbol$pch,
+                     col = plot.symbol$col,
+                     cex = plot.symbol$cex,
+                     fontfamily = plot.symbol$fontfamily,
+                     fontface = chooseFace(plot.symbol$fontface, plot.symbol$font),
+                     fontsize = fontsize.points)
     }
     else
     {
-
-        maxn <- max(by(y, x, length)) ## used if varwidth = TRUE
-
-        xscale <- current.viewport()$xscale
-        lower <- ceiling(xscale[1])
-        width <- box.ratio/(1+box.ratio)
-        yscale <- current.viewport()$yscale
-
-        for (xval in levels.fos) {
-            ##xval  <- i
-            stats <- boxplot.stats(y[x==xval], coef = coef, do.out = do.out)
-
-            if (stats$n>0)
-            {
-                pushViewport(viewport(x = unit(xval, "native"),
-                                      width = unit((if (varwidth)
-                                      sqrt(stats$n/maxn)  else 1) * width, "native"),
-                                      yscale = yscale))
-                
-                r.x <- (stats$stats[2]+stats$stats[4])/2
-                r.w <- stats$stats[4]-stats$stats[2]
-                grid.rect(y = unit(r.x, "native"), height = unit(r.w, "native"),
-                          gp = gpar(lwd = box.rectangle$lwd,
-                          lty = box.rectangle$lty,
-                          fill = fill,
-                          col = box.rectangle$col))
-                
-                grid.lines(y = unit(stats$stats[1:2],"native"),
-                           x = unit(c(.5,.5), "npc"),
-                           gp = gpar(col = box.umbrella$col,
-                           lwd = box.umbrella$lwd,
-                           lty = box.umbrella$lty))
-                
-                grid.lines(y = unit(stats$stats[4:5],"native"),
-                           x = unit(c(.5,.5), "npc"),
-                           gp = gpar(col = box.umbrella$col,
-                           lwd = box.umbrella$lwd,
-                           lty = box.umbrella$lty))
-                
-                grid.lines(y = unit(rep(stats$stats[1],2),"native"),
-                           x = unit(c(0,1), "npc"),
-                           gp = gpar(col = box.umbrella$col,
-                           lwd = box.umbrella$lwd,
-                           lty = box.umbrella$lty))
-                
-                grid.lines(y = unit(rep(stats$stats[5],2),"native"),
-                           x = unit(c(0,1), "npc"),
-                           gp = gpar(col = box.umbrella$col,
-                           lwd = box.umbrella$lwd,
-                           lty = box.umbrella$lty))
-                
-                grid.points(y = stats$stats[3], x = .5, pch = pch, 
-                            gp =
-                            gpar(col = col, cex = cex,
-                                 fontfamily = fontfamily,
-                                 fontface = chooseFace(fontface, font),
-                                 fontsize = fontsize.points))
-                
-                if ((l<-length(stats$out))>0)
-                    grid.points(y = stats$out, x = rep(.5,l),
-                                pch = plot.symbol$pch,
-                                gp =
-                                gpar(col = plot.symbol$col,
-                                     cex = plot.symbol$cex,
-                                     fontfamily = plot.symbol$fontfamily,
-                                     fontface = chooseFace(plot.symbol$fontface, plot.symbol$font),
-                                     fontsize = fontsize.points))
-                
-                popViewport()
-                
-            }
+        blist <-
+            tapply(y, factor(x, levels = levels.fos),
+                   boxplot.stats,
+                   coef = coef,
+                   do.out = do.out)
+        blist.stats <- t(sapply(blist, "[[", "stats"))
+        blist.out <- lapply(blist, "[[", "out")
+        blist.height <- box.ratio / (1 + box.ratio)
+        if (varwidth)
+        {
+            maxn <- max(by(y, x, length))
+            blist.n <- sapply(blist, "[[", "n")
+            blist.height <- sqrt(blist.n / maxn) * blist.height
         }
-        
+
+        ## box
+
+        panel.rect(ybottom = blist.stats[, 2],
+                   ytop = blist.stats[, 4],
+                   x = levels.fos, 
+                   width = blist.height,
+                   lwd = box.rectangle$lwd,
+                   lty = box.rectangle$lty,
+                   fill = fill,
+                   col = box.rectangle$col)
+
+        ## whiskers
+
+        panel.segments(rep(levels.fos, 2),
+                       c(blist.stats[, 2], blist.stats[, 4]),
+                       rep(levels.fos, 2),
+                       c(blist.stats[, 1], blist.stats[, 5]),
+                       col = box.umbrella$col,
+                       lwd = box.umbrella$lwd,
+                       lty = box.umbrella$lty)
+        panel.segments(levels.fos - blist.height / 2,
+                       c(blist.stats[, 1], blist.stats[, 5]),
+                       levels.fos + blist.height / 2,
+                       c(blist.stats[, 1], blist.stats[, 5]),
+                       col = box.umbrella$col,
+                       lwd = box.umbrella$lwd,
+                       lty = box.umbrella$lty)
+
+        ## dot
+
+        if (all(pch == "|"))
+        {
+            panel.segments(levels.fos - blist.height / 2,
+                           blist.stats[, 3],
+                           levels.fos + blist.height / 2,
+                           blist.stats[, 3],
+                           lwd = box.rectangle$lwd,
+                           lty = box.rectangle$lty,
+                           col = box.rectangle$col)
+        }
+        else
+        {
+            panel.points(x = levels.fos,
+                         y = blist.stats[, 3],
+                         pch = pch,
+                         col = col, cex = cex,
+                         fontfamily = fontfamily,
+                         fontface = chooseFace(fontface, font),
+                         fontsize = fontsize.points)
+        }
+
+        ## outliers
+
+        panel.points(x = rep(levels.fos, sapply(blist.out, length)),
+                     y = unlist(blist.out),
+                     pch = plot.symbol$pch,
+                     col = plot.symbol$col,
+                     cex = plot.symbol$cex,
+                     fontfamily = plot.symbol$fontfamily,
+                     fontface = chooseFace(plot.symbol$fontface, plot.symbol$font),
+                     fontsize = fontsize.points)
     }
 }
 
+
+
+## panel.bwplot.old <-
+##     function(x, y, box.ratio = 1,
+##              horizontal = TRUE,
+##              pch = box.dot$pch,
+##              col = box.dot$col,
+##              cex = box.dot$cex,
+##              font = box.dot$font,
+##              fontfamily = box.dot$fontfamily,
+##              fontface = box.dot$fontface, 
+##              fill = box.rectangle$fill,
+##              varwidth = FALSE,
+##              levels.fos = if (horizontal) unique(y) else unique(x),
+##              coef = 1.5, do.out = TRUE, ...)
+## {
+##     x <- as.numeric(x)
+##     y <- as.numeric(y)
+
+##     if (length(x) < 1) return()
+
+##     box.dot <- trellis.par.get("box.dot")
+##     box.rectangle <- trellis.par.get("box.rectangle")
+##     box.umbrella <- trellis.par.get("box.umbrella")
+##     plot.symbol <- trellis.par.get("plot.symbol")
+
+##     fontsize.points <- trellis.par.get("fontsize")$points
+##     xscale <- current.viewport()$xscale
+##     yscale <- current.viewport()$yscale
+
+##     if (horizontal)
+##     {
+
+##         maxn <- max(by(x, y, length)) ## used if varwidth = TRUE
+##         ##lower <- ceiling(yscale[1])
+##         height <- box.ratio/(1+box.ratio)
+
+##         for (yval in levels.fos)
+##         {
+
+##             ## yval  <- i
+##             stats <- boxplot.stats(x[y==yval], coef = coef, do.out = do.out)
+            
+##             if (stats$n>0)
+##             {
+##                 pushViewport(viewport(y=unit(yval, "native"),
+##                                       height = unit((if (varwidth)
+##                                       sqrt(stats$n/maxn)  else 1) * height, "native"),
+##                                       xscale = xscale))
+                
+##                 r.x <- (stats$stats[2]+stats$stats[4])/2
+##                 r.w <- stats$stats[4]-stats$stats[2]
+##                 grid.rect(x = unit(r.x, "native"), width = unit(r.w, "native"),
+##                           gp = gpar(lwd = box.rectangle$lwd,
+##                           lty = box.rectangle$lty,
+##                           fill = fill,
+##                           col = box.rectangle$col))
+                
+##                 grid.lines(x = unit(stats$stats[1:2],"native"),
+##                            y=unit(c(.5,.5), "npc"),
+##                            gp = gpar(col = box.umbrella$col,
+##                            lwd = box.umbrella$lwd,
+##                            lty = box.umbrella$lty))
+                
+##                 grid.lines(x = unit(stats$stats[4:5],"native"),
+##                            y=unit(c(.5,.5), "npc"),
+##                            gp = gpar(col = box.umbrella$col,
+##                            lwd = box.umbrella$lwd,
+##                            lty = box.umbrella$lty))
+                
+##                 grid.lines(x = unit(rep(stats$stats[1],2),"native"),
+##                            y=unit(c(0,1), "npc"),
+##                            gp = gpar(col = box.umbrella$col,
+##                            lwd = box.umbrella$lwd,
+##                            lty = box.umbrella$lty))
+                
+##                 grid.lines(x = unit(rep(stats$stats[5],2),"native"),
+##                            y=unit(c(0,1), "npc"),
+##                            gp = gpar(col = box.umbrella$col,
+##                            lwd = box.umbrella$lwd,
+##                            lty = box.umbrella$lty))
+                
+##                 grid.points(x = stats$stats[3], y = .5, pch = pch, 
+##                             gp =
+##                             gpar(col = col, cex = cex,
+##                                  fontfamily = fontfamily,
+##                                  fontface = chooseFace(fontface, font),
+##                                  fontsize = fontsize.points))
+                
+##                 if ((l<-length(stats$out))>0)
+##                     grid.points(x = stats$out, y = rep(.5,l),
+##                                 pch = plot.symbol$pch,
+##                                 gp =
+##                                 gpar(col = plot.symbol$col,
+##                                      cex = plot.symbol$cex,
+##                                      fontfamily = plot.symbol$fontfamily,
+##                                      fontface = chooseFace(plot.symbol$fontface, plot.symbol$font),
+##                                      fontsize = fontsize.points))
+                
+##                 popViewport()
+                
+##             }
+##         }
+        
+##     }
+##     else
+##     {
+
+##         maxn <- max(by(y, x, length)) ## used if varwidth = TRUE
+##         ##lower <- ceiling(xscale[1])
+##         width <- box.ratio/(1+box.ratio)
+
+##         for (xval in levels.fos)
+##         {
+##             ##xval  <- i
+##             stats <- boxplot.stats(y[x==xval], coef = coef, do.out = do.out)
+
+##             if (stats$n>0)
+##             {
+##                 pushViewport(viewport(x = unit(xval, "native"),
+##                                       width = unit((if (varwidth)
+##                                       sqrt(stats$n/maxn)  else 1) * width, "native"),
+##                                       yscale = yscale))
+                
+##                 r.x <- (stats$stats[2]+stats$stats[4])/2
+##                 r.w <- stats$stats[4]-stats$stats[2]
+##                 grid.rect(y = unit(r.x, "native"), height = unit(r.w, "native"),
+##                           gp = gpar(lwd = box.rectangle$lwd,
+##                           lty = box.rectangle$lty,
+##                           fill = fill,
+##                           col = box.rectangle$col))
+                
+##                 grid.lines(y = unit(stats$stats[1:2],"native"),
+##                            x = unit(c(.5,.5), "npc"),
+##                            gp = gpar(col = box.umbrella$col,
+##                            lwd = box.umbrella$lwd,
+##                            lty = box.umbrella$lty))
+                
+##                 grid.lines(y = unit(stats$stats[4:5],"native"),
+##                            x = unit(c(.5,.5), "npc"),
+##                            gp = gpar(col = box.umbrella$col,
+##                            lwd = box.umbrella$lwd,
+##                            lty = box.umbrella$lty))
+                
+##                 grid.lines(y = unit(rep(stats$stats[1],2),"native"),
+##                            x = unit(c(0,1), "npc"),
+##                            gp = gpar(col = box.umbrella$col,
+##                            lwd = box.umbrella$lwd,
+##                            lty = box.umbrella$lty))
+                
+##                 grid.lines(y = unit(rep(stats$stats[5],2),"native"),
+##                            x = unit(c(0,1), "npc"),
+##                            gp = gpar(col = box.umbrella$col,
+##                            lwd = box.umbrella$lwd,
+##                            lty = box.umbrella$lty))
+                
+##                 grid.points(y = stats$stats[3], x = .5, pch = pch, 
+##                             gp =
+##                             gpar(col = col, cex = cex,
+##                                  fontfamily = fontfamily,
+##                                  fontface = chooseFace(fontface, font),
+##                                  fontsize = fontsize.points))
+                
+##                 if ((l<-length(stats$out))>0)
+##                     grid.points(y = stats$out, x = rep(.5,l),
+##                                 pch = plot.symbol$pch,
+##                                 gp =
+##                                 gpar(col = plot.symbol$col,
+##                                      cex = plot.symbol$cex,
+##                                      fontfamily = plot.symbol$fontfamily,
+##                                      fontface = chooseFace(plot.symbol$fontface, plot.symbol$font),
+##                                      fontsize = fontsize.points))
+                
+##                 popViewport()
+                
+##             }
+##         }
+        
+##     }
+## }
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+panel.violin <-
+    function(x, y, box.ratio = 1, horizontal = TRUE,
+
+             alpha = bar.fill$alpha,
+             border = bar.fill$border,
+             lty = bar.fill$lty,
+             lwd = bar.fill$lwd,
+             col = bar.fill$col,
+
+             varwidth = FALSE,
+
+             bw = NULL,
+             adjust = NULL,
+             kernel = NULL,
+             window = NULL,
+             width = NULL,
+             n = 50,
+             from = NULL,
+             to = NULL,
+             cut = NULL,
+             na.rm = TRUE,
+             
+             ...)
+{
+    x <- as.numeric(x)
+    y <- as.numeric(y)
+
+    if (length(x) < 1) return()
+
+    ##reference.line <- trellis.par.get("reference.line")
+    bar.fill <- trellis.par.get("bar.fill")
+
+    ## density doesn't handle unrecognized arguments (not even to
+    ## ignore it).  A tedious but effective way to handle that is to
+    ## have all arguments to density be formal arguments to this panel
+    ## function, as follows:
+
+    darg <- list()
+    darg$bw <- bw
+    darg$adjust <- adjust
+    darg$kernel <- kernel
+    darg$window <- window
+    darg$width <- width
+    darg$n <- n
+    darg$from <- from
+    darg$to <- to
+    darg$cut <- cut
+    darg$na.rm <- na.rm
+
+    my.density <- function(x) do.call("density", c(list(x = x), darg))
+
+    numeric.list <- if (horizontal) split(x, factor(y)) else split(y, factor(x))
+    levels.fos <- as.numeric(names(numeric.list))
+    d.list <- lapply(numeric.list, my.density)
+    ## n.list <- sapply(numeric.list, length)  UNNECESSARY
+    dx.list <- lapply(d.list, "[[", "x")
+    dy.list <- lapply(d.list, "[[", "y")
+
+    max.d <- sapply(dy.list, max)
+    if (varwidth) max.d[] <- max(max.d)
+
+    ##str(max.d)
+    
+    xscale <- current.viewport()$xscale
+    yscale <- current.viewport()$yscale
+    height <- box.ratio / (1 + box.ratio)
+
+    if (horizontal)
+    {
+        for (i in seq(along = levels.fos))
+        {
+            pushViewport(viewport(y = unit(levels.fos[i], "native"),
+                                  height = unit(height, "native"),
+                                  yscale = c(max.d[i] * c(-1, 1)),
+                                  xscale = xscale))
+            grid.polygon(x = c(dx.list[[i]], rev(dx.list[[i]])),
+                         y = c(dy.list[[i]], -rev(dy.list[[i]])),
+                         default.units = "native",
+                         gp = gpar(fill = col, col = border, lty = lty, lwd = lwd, alpha = alpha))
+            popViewport()
+        }
+    }
+    else
+    {
+        for (i in seq(along = levels.fos))
+        {
+            pushViewport(viewport(x = unit(levels.fos[i], "native"),
+                                  width = unit(height, "native"),
+                                  xscale = c(max.d[i] * c(-1, 1)),
+                                  yscale = yscale))
+            grid.polygon(y = c(dx.list[[i]], rev(dx.list[[i]])),
+                         x = c(dy.list[[i]], -rev(dy.list[[i]])),
+                         default.units = "native",
+                         gp = gpar(fill = col, col = border, lty = lty, lwd = lwd, alpha = alpha))
+            popViewport()
+        }
+    }
+    invisible()
+}
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
 
 
 
