@@ -1,5 +1,5 @@
-### Copyright 2001-2004  Deepayan Sarkar <deepayan@stat.wisc.edu>
-### Copyright 2001-2003  Saikat DebRoy <saikat@stat.wisc.edu>
+### Copyright (C) 2001-2005  Deepayan Sarkar <Deepayan.Sarkar@R-project.org>
+### Copyright (C) 2001-2005  Saikat DebRoy <saikat@stat.wisc.edu>
 ###
 ### This file is part of the lattice library for R.
 ### It is made available under the terms of the GNU General Public
@@ -210,7 +210,8 @@ latticeParseFormula <-
         nRHS <- 1
     }
 
-    if (length(model) == 3) {
+    if (length(model) == 3)
+    {
 
         ## Note: special case if tmp is a matrix. This probably means
         ## we are dealing with the parametric 3-D surface rendering
@@ -234,7 +235,8 @@ latticeParseFormula <-
                                })), nRHS)
     }
 
-    if (dimension == 2) {
+    if (dimension == 2)
+    {
 
         ## this part belongs in the first if block, but we need nobs
         ## later in either case, so keeping it here.
@@ -273,7 +275,8 @@ latticeParseFormula <-
         nRows <- length(ans$right)/(nLHS * nRHS)
     }
     else if (dimension == 3 && length(modelRHS) == 3 &&
-             (modelRHS[[1]] == "*" || modelRHS[[1]] == "+")) {
+             (modelRHS[[1]] == "*" || modelRHS[[1]] == "+"))
+    {
 
         ## Note that when dimension = 3, multiple does not apply to
         ## RHS, but nLHS may be > 1
